-- Frames
local playerFrame = CreateFrame("Frame", "DLUPlayerFrame");
local professionFrame = CreateFrame("Frame", "DLUProfessionFrame");

-- Variables
local professionsDetected = false;
local maxLevel = GetMaxPlayerLevel();
local class, classFileName = UnitClass("player");
local factionGroup, factionName = UnitFactionGroup("player");

local expansionProgressionTable = {};
expansionProgressionTable[LE_EXPANSION_CLASSIC] = 58;
expansionProgressionTable[LE_EXPANSION_BURNING_CRUSADE] = 68;
expansionProgressionTable[LE_EXPANSION_WRATH_OF_THE_LICH_KING] = 80;
expansionProgressionTable[LE_EXPANSION_CATACLYSM] = 85;
expansionProgressionTable[LE_EXPANSION_MISTS_OF_PANDARIA] = 90;
expansionProgressionTable[LE_EXPANSION_WARLORDS_OF_DRAENOR] = 100;
expansionProgressionTable[LE_EXPANSION_LEGION] = 110;

-- Race based tables
local orcTable = {
	[1] = 889,
	[5] = 4
};
local trollTable = {

};

local dwarfTable = {
	[1] = 866,
	[6] = 27,
	[10] = 35,
	[20] = 40,
	[25] = 16,
	[30] = 26
};

-- Continent based tables
local kalimdorTable = {
	[10] = 11,
	[20] = 43,
	[25] = 81,
	[30] = 101,
	[35] = 121,
	[40] = 61,
	[45] = 161,
	[50] = 201,
	[55] = 261
};
local easternKingdomTable = {

};

-- Expansion based tables
local tbcTable = {
	[58] = 465
};
local wotlkTable = {
	[68] = 486
};
local cataTable = {
	[80] = 606
};
local mopTable = {
	[85] = 806
};
local wodTable = {
	[90] = 941
};
local legionTable = {
	[100] = 1007
};

-- Expansion level scaling
--[[
TBC and Wrath: 60-80
Cata and Mists: 80-90
]]

-- Faction based tables
local hordeLevelingTable = {};
local alliLevelingTable = {};

-- Setting globals
DLU.player.faction = factionName;
DLU.player.level = UnitLevel("player");

function DLU.player.getRidingSkill()
	local riding_data = {33388,33391,34090,34091,90265};
	local riding_skill = 0;
	for i=#riding_data, 1, -1 do
		if (IsSpellKnown(riding_data[i])) then
			riding_skill = i*75;
			break;
		end
	end

	return riding_skill;
end

-- Helper functions
local function mergeTables(t1, t2)
	for k,v in pairs(t2) do
		t1[k] = v;
	end
end

local function getFactionColorId(factionName)
	local factionId = nil;

	if (factionGroup == "Horde") then
		factionId = 0;
	elseif (factionGroup == "Alliance") then
		factionId = 1;
	end

	return factionId;
end

local function getFactionColor(factionName)
	local factionColorId = getFactionColorId(factionName);
	local factionString = nil;
	if (factionColorId ~= nil) then
		factionString = "|cff" .. DLU.RGBPercToHex(PLAYER_FACTION_COLORS[factionColorId].r, PLAYER_FACTION_COLORS[factionColorId].g, PLAYER_FACTION_COLORS[factionColorId].b) .. factionName .. "|r";
	else
		if (factionName ~= "") then
			factionString = factionName;
		else
			factionString = factionGroup;
		end
	end

	return factionString;
end

local function checkLevelingTable()
	for level, zoneId in pairs(levelingTable) do
		local zoneName = GetMapNameByID(zoneId);
		print(format("%s (%i)", zoneName, level));
	end
end

-- Player info --
local characterInfo = {
	name = UnitName("player"),
	gender = DLU.genderTable[UnitSex("player")],
	class = classFileName,
	classWithColor = "|c" .. RAID_CLASS_COLORS[classFileName].colorStr .. class .. "|r",
	race = UnitRace("player"),
	faction = getFactionColor(factionName)
}

-- Functions
local function getExpansionName()
	local expansion = GetExpansionLevel()+1;
	local expansionTable = {"WoW: Vanilla", "WoW: The Burning Crusade", "WoW: Wrath of the Lich King", "WoW: Cataclysm", "WoW: Mists of Pandaria", "WoW: Warlords of Draenor", "WoW: Legion"};

	local expansionName = "Unknown";
	if (expansionTable[expansion] ~= nil) then
		expansionName = expansionTable[expansion];
	end

	return expansionName;
end

local function HelloPlayer()
	local text = nil;

	local factionExpression = characterInfo.faction;
	if (characterInfo.faction ~= "Neutral") then
		factionExpression = "the " .. factionExpression;
	end

	if (DLU.player.level < maxLevel) then
		text = string.format("Hello, %s. You are a level %i %s %s %s from %s. Your current max level is %i (%s).", characterInfo.name, DLU.player.level, characterInfo.gender, characterInfo.race, characterInfo.classWithColor, factionExpression, maxLevel, getExpansionName());
	elseif (DLU.player.level == maxLevel and CanUpgradeExpansion()) then
		text = string.format("Hello, %s. You are a %s %s %s from %s, and you are currently at your max level (level %i). You can upgrade the game to continue your leveling adventure...", characterInfo.name, characterInfo.gender, characterInfo.race, characterInfo.classWithColor, factionExpression, DLU.player.level);
	else
		text = string.format("Hello, %s. You are a %s %s %s from %s, and you are currently at your max level (level %i).", characterInfo.name, characterInfo.gender, characterInfo.race, characterInfo.classWithColor, factionExpression, DLU.player.level);
	end
	print(text);
end

local function playerLevelUp(self, event, ...)
	local newLevel = ...;
	DLU.player.level = newLevel;

	local factionId = getFactionColorId(DLU.player.faction);
	local _, race = UnitRace("player");
	local expansionLevel = GetExpansionLevel();
	local currentContinent = GetCurrentMapContinent();

	local levelingTable = {};

	if (factionId == 0) then
		if (race == "Orc") then
			mergeTables(levelingTable, orcTable);
		end
	elseif (factionId == 1) then
		if (race == "Dwarf") then
			mergeTables(levelingTable, dwarfTable);
		elseif (race == "Gnome") then
			-- Start the more or less at the same location as the dwarves
			mergeTables(levelingTable, dwarfTable);
		end
	end

	if (currentContinent == 1) then
		mergeTables(levelingTable, kalimdorTable);
	elseif (currentContinent == 2) then
		mergeTables(levelingTable, easternKingdomTable);
	end

	if (expansionLevel >= 2) then
		mergeTables(levelingTable, tbcTable);
	end
	if (expansionLevel >= 3) then
		mergeTables(levelingTable, wotlkTable);
	end
	if (expansionLevel >= 4) then
		mergeTables(levelingTable, cataTable);
	end
	if (expansionLevel >= 5) then
		mergeTables(levelingTable, mopTable);
	end
	if (expansionLevel >= 6) then
		mergeTables(levelingTable, wodTable);
	end
	if (expansionLevel >= 7) then
		mergeTables(levelingTable, legionTable);
	end

	for recommededLevel, zoneId in pairs(levelingTable) do
		if (recommededLevel == newLevel) then
			local zoneName = GetMapNameByID(zoneId);
			local message = format("You might want go to %s now ;-)", zoneName);
			DLU.createNotification(message, "leveling");
		end
	end
end

-- Events
playerFrame:RegisterEvent("PLAYER_LEVEL_UP");
playerFrame:SetScript("OnEvent", playerLevelUp);

-- Startup Event
if (IsAddOnLoaded(DLU.addonName)) then
	HelloPlayer();
end
